/***********************************************************
 *
 * d_Display_DMA2D.h
 * SYSTART GmbH
 * JOH, 27.11.2018
 *
 ***********************************************************/
#ifndef DRIVER_D_DISPLAY_DMA2D_H_
#define DRIVER_D_DISPLAY_DMA2D_H_

/***********************************************************
 * Includes
 ***********************************************************/
#include "global_Display_Touch.h"
#include "stm32f4xx.h"
#include "driver/d_Fonts.h"
#include "driver/d_SDRAM_FLASH.h"

/***********************************************************
 * Defines
 ***********************************************************/
#define LAYER_2_ACTIVE

/***********************************************************
 * Public variables
 ***********************************************************/
extern tFont courier_new_15_2;
extern tFont courier_new_14;
extern tFont arial_28_deg;
extern tImage ebssystart_logo_35;
extern tImage twoInone_display_logo;
extern tImage Display21_Logo_Y_43Inch;

/***********************************************************
 * Function declarations
************************************************************/

/**
* @brief Initializes display and DMA2D
*
*/
void DMA2D_Display_init();

/**
* @brief Switches frontbuffer of layer
*
* @param layer Layer number (Layer_1, Layer_2)
* @param buffer Buffer number (Buffer_1, Buffer_2)
*
*/
void switch_buffer(Layer_e layer, Buffer_e buffer);

/**
* @brief Swaps frontbuffer of layer
*
* @param layer Layer number (Layer_1, Layer_2)
*
* @return backbuffer
*
*/
Buffer_e swap_buffer(Layer_e layer);

/**
* @brief Sets alpha value for a layer
*
* @param layer Layer number (Layer_1, Layer_2)
* @param alpha Alpha value (transparency)
* (0 = transparent, 0xFF = opak)
*
*/
void set_layer_alpha(Layer_e layer, uint8_t alpha);

/**
* @brief Draw a single pixel
*
* @param xpos 		x-position
* @param ypos 		y-position
* @param color 		Color
* @param layer		Layer number (Layer_1, Layer_2)
* @param buffer		Buffer number (Buffer_1, Buffer_2)
*
*/
void DMA2D_DrawPixel(uint16_t xpos, uint16_t ypos, uint32_t color, Layer_e layer, Buffer_e buffer);

/**
* @brief Fill whole buffer with a color
*
* @param color 		Color
* @param layer		Layer number (Layer_1, Layer_2)
* @param buffer		Buffer number (Buffer_1, Buffer_2)
*
*/
void DMA2D_Fill_Color(uint32_t color, Layer_e layer, Buffer_e buffer);

/**
* @brief Fill whole buffer with an image
*
* @param addr_image 	Address of the image
* @param source_format	Color format of the image (CM_RGB565, CM_RGB888, ...)
* @param layer			Layer number (Layer_1, Layer_2)
* @param buffer			Buffer number (Buffer_1, Buffer_2)
*
*/
void DMA2D_Fill_Image(uint32_t addr_image, uint32_t source_format, Layer_e layer, Buffer_e buffer);

/**
* @brief Fill part of buffer with an image
*
* @param xpos 			x-position
* @param ypos 			y-position
* @param xsize 			Width of image
* @param ysize			Height of image
* @param alpha			Alpha value
* @param alpha_mode		Alpha mode
* @param addr_image 	Address of the image
* @param source_format 	Color format of the image (CM_RGB565, CM_RGB888, ...)
* @param layer			Layer number (Layer_1, Layer_2)
* @param buffer			Buffer number (Buffer_1, Buffer_2)
*
*/
void DMA2D_Draw_Image(uint16_t xpos, uint16_t ypos, uint16_t xsize, uint16_t ysize, uint8_t alpha, uint32_t alpha_mode, uint32_t addr_image, uint32_t source_format, Layer_e layer, Buffer_e buffer);

/**
* @brief Draw horizontal line
*
* @param xpos 		x-position
* @param ypos 		y-position
* @param length 	Length of line
* @param width		Width of line
* @param color		Color of line
* @param layer		Layer number (Layer_1, Layer_2)
* @param buffer		Buffer number (Buffer_1, Buffer_2)
*
*/
void DMA2D_Draw_X_Line(uint16_t xpos, uint16_t ypos, uint16_t length, uint16_t width, uint32_t color, Layer_e layer, Buffer_e buffer);

/**
* @brief Draw vertical line
*
* @param xpos 		x-position
* @param ypos 		y-position
* @param length 	Length of line
* @param width		Width of line
* @param color		Color of line
* @param layer		Layer number (Layer_1, Layer_2)
* @param buffer		Buffer number (Buffer_1, Buffer_2)
*
*/
void DMA2D_Draw_Y_Line(uint16_t xpos, uint16_t ypos, uint16_t length, uint16_t width, uint32_t color, Layer_e layer, Buffer_e buffer);




void DMA2D_Draw_Line(uint16_t x0, uint16_t y0, uint16_t x1, uint16_t y1, uint32_t color, Layer_e layer, Buffer_e buffer);


/**
* @brief Draw a filled rectangle
*
* @param xpos 		x-position
* @param ypos 		y-position
* @param length 	Length of rectangle in pixels
* @param height		Height of rectangle in pixels
* @param color		Color of rectangle
* @param layer		Layer number (Layer_1, Layer_2)
* @param buffer		Buffer number (Buffer_1, Buffer_2)
*
*/
void DMA2D_Draw_FilledRectangle(uint16_t xpos, uint16_t ypos, uint16_t length, uint16_t height, uint32_t color, Layer_e layer, Buffer_e buffer);

/**
* @brief Draw a rectangle
*
* @param xpos 		x-position
* @param ypos 		y-position
* @param length 	Length of rectangle in pixels
* @param height		Height of rectangle in pixels
* @param color		Color of rectangle
* @param layer		Layer number (Layer_1, Layer_2)
* @param buffer		Buffer number (Buffer_1, Buffer_2)
*
*/
void DMA2D_Draw_Rectangle(uint16_t xpos, uint16_t ypos, uint16_t length, uint16_t height, uint32_t color, uint8_t thickness, Layer_e layer, Buffer_e buffer);

/**
* @brief Draw a single character
*
* @param sign		Character to be written
* @param xpos 		x-position
* @param ypos 		y-position
* @param color		Font color
* @param font		Font
* @param layer	Layer number (Layer_1, Layer_2) Foreground
* @param buffer	Buffer number (Buffer_1, Buffer_2) Foreground
* @param bglayer	Layer number (Layer_1, Layer_2) Background
* @param bgbuffer	Buffer number (Buffer_1, Buffer_2) Background
*
* @return x_offset	Width of written sign
*
*/
uint16_t DMA2D_write_char(char sign, uint16_t xpos, uint16_t ypos, uint32_t color, tFont* font, Layer_e layer, Buffer_e buffer, Layer_e bglayer, Buffer_e bgbuffer);

/**
* @brief Draw text
*
* @param string		Text to be written
* @param xpos 		x-position
* @param ypos 		y-position
* @param color		Font color
* @param font		Font
* @param layer		Layer number (Layer_1, Layer_2) Foreground
* @param buffer		Buffer number (Buffer_1, Buffer_2) Foreground
* @param bglayer	Layer number (Layer_1, Layer_2) Background
* @param bgbuffer	Buffer number (Buffer_1, Buffer_2) Background
* @return Length of string
*/
uint16_t DMA2D_write_string(char* string, uint16_t xpos, uint16_t ypos, uint32_t color, tFont* font, Layer_e layer, Buffer_e buffer, Layer_e bglayer, Buffer_e bgbuffer);

uint16_t get_string_length(char* string, tFont* font);

#endif /* DRIVER_D_DISPLAY_DMA2D_H_ */
