/*****************************************************************
 *  File: uart_rs232_comm.c
 *
 *  Summary: Functions to send, receive and parse uart data
 *
 ****************************************************************/
 
/***********************************************************
 * Includes
 ***********************************************************/
#include <application/uart_esp32_comm.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "driver/d_uart.h"
#include "driver/d_Display_DMA2D.h"

union float_bytes_union{
  uint8_t bytes[4];
  float floatVal;
};

/***********************************************************
 * Public variables
 ***********************************************************/
//static weather_data_struct weather_data;
static union float_bytes_union float_bytes;




/*****************************************************************
 *
 * Function: comm_err_t send_cmd_esp32(uint8_t cmd)
 *
 * Summary: Sends a one-byte command to the esp32 module
 *
 * Parameters:
 *     cmd - one-byte command
 *
 * Return:
 *  0 - if success
 *
 * Revision History:
 * Creation Date: 28062021
 *
 *****************************************************************/
comm_err_t send_cmd_esp32(uint8_t cmd) {

	tTimer send_timeout;

	send_timeout = timer_get_time_ms();
	uart_SendByte(COM3, cmd);
	if(timer_get_time_ms() - send_timeout > 10) return FCN_TIMEOUT;

	return FCN_OK;
}

/*****************************************************************
 *
 * Function: comm_err_t get_data_esp32(weather_data_struct* w_data)
 *
 * Summary: Get the temperature, humidity, pressure,
 * approximate altitude and gas resistance
 * from bme680 sensor via esp32
 *
 * Parameters:
 *
 * Return:
 *  0 - if success
 *
 * Revision History:
 * Creation Date: 28062021
 *
 *****************************************************************/
comm_err_t get_data_esp32(weather_data_struct* w_data) {
	uint8_t data[4];
	//temperature
	while(uart_Receive4Byte(COM3, &data[0], BIG_ENDIAN_T) == 0);
	float_bytes.bytes[0] = data[0];
	float_bytes.bytes[1] = data[1];
	float_bytes.bytes[2] = data[2];
	float_bytes.bytes[3] = data[3];
	w_data->temperature = float_bytes.floatVal;
	//humidity
	while(uart_Receive4Byte(COM3, &data[0], BIG_ENDIAN_T) == 0);
	float_bytes.bytes[0] = data[0];
	float_bytes.bytes[1] = data[1];
	float_bytes.bytes[2] = data[2];
	float_bytes.bytes[3] = data[3];
	w_data->humidity = float_bytes.floatVal;
	//pressure
	while(uart_Receive4Byte(COM3, &data[0], BIG_ENDIAN_T) == 0);
	float_bytes.bytes[0] = data[0];
	float_bytes.bytes[1] = data[1];
	float_bytes.bytes[2] = data[2];
	float_bytes.bytes[3] = data[3];
	w_data->pressure = float_bytes.floatVal;
	//approx altitude
	while(uart_Receive4Byte(COM3, &data[0], BIG_ENDIAN_T) == 0);
	float_bytes.bytes[0] = data[0];
	float_bytes.bytes[1] = data[1];
	float_bytes.bytes[2] = data[2];
	float_bytes.bytes[3] = data[3];
	w_data->approx_altitude = float_bytes.floatVal;
	//gas resistance
	while(uart_Receive4Byte(COM3, &data[0], BIG_ENDIAN_T) == 0);
	float_bytes.bytes[0] = data[0];
	float_bytes.bytes[1] = data[1];
	float_bytes.bytes[2] = data[2];
	float_bytes.bytes[3] = data[3];
	w_data->gas_resistance = float_bytes.floatVal;

	return FCN_OK;
}

comm_err_t get_temperature_esp32(weather_data_struct* w_data) {
	uint8_t data[4];

	while(uart_Receive4Byte(COM3, &data[0], BIG_ENDIAN_T) == 0);

	float_bytes.bytes[0] = data[0];
	float_bytes.bytes[1] = data[1];
	float_bytes.bytes[2] = data[2];
	float_bytes.bytes[3] = data[3];

	w_data->temperature = float_bytes.floatVal;
	return FCN_OK;
}

/*****************************************************************
 *
 * Function: comm_err_t check_communication_esp32(void)
 *
 * Summary: Checks if the UART communication between STM32 and ESP32 chip is established
 *
 * Parameters:
 *
 *
 * Return:
 *  0 - if success
 *
 * Revision History:
 * Creation Date: 28062021
 *
 *****************************************************************/
comm_err_t check_communication_esp32(void){
	uint8_t response;


	comm_err_t error = send_cmd_esp32((uint8_t)CHECK_CONNECTION);
	if(error == FCN_TIMEOUT){
		return error;
	}
	delay_ms(1000);

	while(uart_Receive1Byte(COM3, &response) == 0);



	if(response == 0xAA){
		return FCN_OK;
	}
	else{
		return FCN_RES_NA;
	}
}

/*****************************************************************
 *
 * Function: comm_err_t get_ip_address_esp32(weather_data_struct* w_data)
 *
 * Summary: Retrieves the IP-Address for the weather-station-webpage from the ESP32
 *
 * Parameters:
 *
 *
 * Return:
 *  0 - if success
 *
 * Revision History:
 * Creation Date: 28062021
 *
 *****************************************************************/
comm_err_t get_ip_address_esp32(weather_data_struct* w_data){
	char ip_string[16];
	comm_err_t error = send_cmd_esp32(GET_IP_ADDRESS);
	if(error == FCN_TIMEOUT){
		return error;
	}
	delay_ms(1000);
	/*UART_RXSTATUS_t string_response = uart_ReceiveString(COM3, ip_string);

	if(string_response != RX_READY){
		return FCN_RES_NA;
	}*/
	uart_ReceiveString(COM3, ip_string);

	strcpy(w_data->ip_address, ip_string);
	return FCN_OK;
}
