/***********************************************************
 *
 * d_uart.c
 * SYSTART GmbH
 * FLB, 12.09.2018
 *
 ***********************************************************/
 
 /***********************************************************
 * Includes
 ***********************************************************/
#include "driver/d_uart.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_usart.h"
#include "misc.h"

//--------------------------------------------------------------
// Definition of all UARTs
// Order like in UART_NAME_t
//--------------------------------------------------------------
UART_t UART[] = {
		// Name, Clock               , AF-UART      ,UART  , Baud , Interrupt
		  {COM1,RCC_APB2Periph_USART1,GPIO_AF_USART1,USART1,115200,USART1_IRQn, // UART1 with 115200 Baud
		// PORT , PIN      , Clock              , Source
		  {GPIOA,GPIO_Pin_10,RCC_AHB1Periph_GPIOA,GPIO_PinSource10},  // TX at PA10
		  {GPIOA,GPIO_Pin_9,RCC_AHB1Periph_GPIOA,GPIO_PinSource9}}, // RX at PA9

		// Name, Clock               , AF-UART      ,UART  , Baud, Interrupt
		  {COM3,RCC_APB1Periph_USART3,GPIO_AF_USART3,USART3,115200,USART3_IRQn, // USART3 with 115200 Baud
		// PORT , PIN      , Clock              , Source
		  {GPIOB,GPIO_Pin_10,RCC_AHB1Periph_GPIOB,GPIO_PinSource10},  // TX at PB10
		  {GPIOB,GPIO_Pin_11,RCC_AHB1Periph_GPIOB,GPIO_PinSource11}}, // RX at PB11

	    // Name, Clock               , AF-UART      ,UART  , Baud , Interrupt
		  {COM6,RCC_APB2Periph_USART6,GPIO_AF_USART6,USART6,115200, USART6_IRQn, // UART6 with 115200 Baud
	    // PORT , PIN      , Clock              , Source
		  {GPIOC,GPIO_Pin_6,RCC_AHB1Periph_GPIOC,GPIO_PinSource6},  // TX at PC6
		  {GPIOC,GPIO_Pin_7,RCC_AHB1Periph_GPIOC,GPIO_PinSource7}}, // RX at PC7

};

 /***********************************************************
 * Declaration internal functions
 ***********************************************************/
void uart_init(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  USART_InitTypeDef USART_InitStructure;
  NVIC_InitTypeDef NVIC_InitStructure;
  UART_NAME_t nr;

  for(nr=0;nr<UART_ANZ;nr++) {

    // Enable clock enable of TX and RX pins
    RCC_AHB1PeriphClockCmd(UART[nr].TX.CLK, ENABLE);
    RCC_AHB1PeriphClockCmd(UART[nr].RX.CLK, ENABLE);

    // Enable clock of UART
    if((UART[nr].UART==USART1) || (UART[nr].UART==USART6)) {
      RCC_APB2PeriphClockCmd(UART[nr].CLK, ENABLE);
    }
    else {
      RCC_APB1PeriphClockCmd(UART[nr].CLK, ENABLE);
    }

    // Connect UART Alternative-Functions with IO-Pins
    GPIO_PinAFConfig(UART[nr].TX.PORT,UART[nr].TX.SOURCE,UART[nr].AF);
    GPIO_PinAFConfig(UART[nr].RX.PORT,UART[nr].RX.SOURCE,UART[nr].AF);

    // UART as Alternative-Function with PushPull
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    // TX-Pin
    GPIO_InitStructure.GPIO_Pin = UART[nr].TX.PIN;
    GPIO_Init(UART[nr].TX.PORT, &GPIO_InitStructure);
    // RX-Pin
    GPIO_InitStructure.GPIO_Pin =  UART[nr].RX.PIN;
    GPIO_Init(UART[nr].RX.PORT, &GPIO_InitStructure);

    // Oversampling
    USART_OverSampling8Cmd(UART[nr].UART, ENABLE);

    // init with Baudrate, 8Databits, 1Stopbit, no parity, no RTS+CTS
    USART_InitStructure.USART_BaudRate = UART[nr].BAUD;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits = USART_StopBits_1;
    USART_InitStructure.USART_Parity = USART_Parity_No;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
    USART_Init(UART[nr].UART, &USART_InitStructure);

    // UART enable
    USART_Cmd(UART[nr].UART, ENABLE);

    // RX-Interrupt enable
    USART_ITConfig(UART[nr].UART, USART_IT_RXNE, ENABLE);

    // enable UART Interrupt-Vector
    NVIC_InitStructure.NVIC_IRQChannel = UART[nr].INT;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);

    //Prepare RX-Buffer
    UART_RX[nr].rx_buffer[0]=RX_END_CHR;
    UART_RX[nr].wr_ptr=0;
    UART_RX[nr].rd_ptr=0;
    UART_RX[nr].status=RX_EMPTY;
  }
}

//--------------------------------------------------------------
// send a string via UART
//--------------------------------------------------------------
void uart_SendString(UART_NAME_t uart, char *ptr, UART_LASTBYTE_t end_cmd)
{
  // send whole string
  while (*ptr != 0) {
    uart_SendByte(uart,*ptr);
    ptr++;
  }
  // if wanted, send end command
  if(end_cmd==LFCR) {
    uart_SendByte(uart,0x0A); // send LineFeed
    uart_SendByte(uart,0x0D); // send CariageReturn
  }
  else if(end_cmd==CRLF) {
    uart_SendByte(uart,0x0D); // send CariageReturn
    uart_SendByte(uart,0x0A); // send LineFeed
  }
  else if(end_cmd==LF) {
    uart_SendByte(uart,0x0A); // send LineFeed
  }
  else if(end_cmd==CR) {
    uart_SendByte(uart,0x0D); // send CariageReturn
  }
}

//--------------------------------------------------------------
// Receive a string via UART
// (the reception is handled via Interrupt)
// Return value :
//  -> if nothing received = RX_EMPTY
//  -> if string received = RX_READY -> String stored in *ptr
//  -> if buffer full      = RX_FULL
//--------------------------------------------------------------
UART_RXSTATUS_t uart_ReceiveString(UART_NAME_t uart, char *ptr)
{
  UART_RXSTATUS_t ret_wert=RX_EMPTY;
  uint16_t rd_pos,wr_pos,n;
  uint8_t wert,ok;

  // read current pointer
  rd_pos=UART_RX[uart].rd_ptr;
  wr_pos=UART_RX[uart].wr_ptr;
  // check if there is data in buffer
  ok=0;
  if(rd_pos!=wr_pos) {
    // search for end
    while(rd_pos!=wr_pos) {
      wert=UART_RX[uart].rx_buffer[rd_pos];
      if(wert==RX_END_CHR) {
        // end detected
        ok=1;
        break;
      }
      // increase read pointer
      rd_pos++;
      if(rd_pos>=RX_BUF_SIZE) rd_pos=0;
    }

    // if end detected, read string
    if(ok==1) {
      // end detected
      ret_wert=RX_READY;
      // read whole string
      rd_pos=UART_RX[uart].rd_ptr;
      n=0;
      while(rd_pos!=wr_pos) {
        wert=UART_RX[uart].rx_buffer[rd_pos];
        // increase read pointer
        rd_pos++;
        if(rd_pos>=RX_BUF_SIZE) rd_pos=0;
        // store only ASCII characters
        if((wert>=RX_FIRST_CHR) && (wert<=RX_LAST_CHR)) {
          ptr[n]=wert;
          n++;
        }
        else if(wert==RX_END_CHR) {
          break;
        }
      }
      // mark end of string
      ptr[n]=0x00;
      // update read pointer
      UART_RX[uart].rd_ptr=rd_pos;
    }
    else {
      // no end detected
      ret_wert=RX_EMPTY;
    }
  }
  else {
    // check for overflow
    if(UART_RX[uart].status==RX_FULL) {
      ret_wert=RX_FULL;
      UART_RX[uart].status=RX_EMPTY;
    }
  }

  return(ret_wert);
}

//--------------------------------------------------------------
// send data-array via UART
// cnt = Number of bytes to be sent
//--------------------------------------------------------------
void uart_SendArray(UART_NAME_t uart, uint8_t *data, uint16_t cnt, Endianness_t endianness)
{
  uint32_t n;

  if(cnt==0) return;

  // send all data
  if(LITTLE_ENDIAN_T == endianness){
	  n = cnt-1;
	  do{
		  uart_SendByte(uart, data[n]);
	  } while(n--);
  }
  else if(BIG_ENDIAN_T == endianness){
	  for(n=0;n<cnt;n++) {
		uart_SendByte(uart,data[n]);
	  }
  }
}

//--------------------------------------------------------------
// receive data-array via UART
// Return value :
//   Number of received bytes [0...RX_BUF_SIZE]
//--------------------------------------------------------------
uint32_t uart_ReceiveArray(UART_NAME_t uart, uint8_t *data)
{
  uint32_t n=0;
  uint16_t rd_pos,wr_pos;
  uint8_t wert;

  // read current pointer
  rd_pos=UART_RX[uart].rd_ptr;
  wr_pos=UART_RX[uart].wr_ptr;
  // check if there is data in buffer
  n=0;
  if(rd_pos!=wr_pos) {
    // read all data
    while(rd_pos!=wr_pos) {
      wert=UART_RX[uart].rx_buffer[rd_pos];
      // increase read pointer
      rd_pos++;
      if(rd_pos>=RX_BUF_SIZE) rd_pos=0;
      // store data
      data[n]=wert;
      n++;
    }
    // update read pointer
    UART_RX[uart].rd_ptr=rd_pos;
  }

  return(n);
}

uint8_t uart_Receive2Byte(UART_NAME_t uart, uint16_t *data, Endianness_t endianness)
{
	uint8_t count = 0;
	uint16_t rd_pos, wr_pos;
	uint8_t wert1, wert2;

	// read current pointer
	rd_pos=UART_RX[uart].rd_ptr;
	wr_pos=UART_RX[uart].wr_ptr;
	// check if there is data in buffer
	if(rd_pos!=wr_pos) {
		//read data
		wert1 = UART_RX[uart].rx_buffer[rd_pos];
		// increase read pointer
		rd_pos++;
		count++;
		if(rd_pos>=RX_BUF_SIZE) rd_pos=0;

		wert2 = UART_RX[uart].rx_buffer[rd_pos];
		rd_pos++;
		count++;
		if(rd_pos>=RX_BUF_SIZE) rd_pos=0;
		if(endianness == BIG_ENDIAN_T) *data = wert1 | wert2 << 8;
		else if(endianness == LITTLE_ENDIAN_T) *data = wert1 << 8 | wert2;
	}
  	if(rd_pos > wr_pos) return 0;
    // update read pointer
    UART_RX[uart].rd_ptr=rd_pos;
    return count;
}

uint8_t uart_Receive4Byte(UART_NAME_t uart, uint8_t *data, Endianness_t endianness)
{
	uint8_t count = 0;
	uint16_t rd_pos, wr_pos;
	uint8_t wert1, wert2, wert3, wert4;

	// read current pointer
	rd_pos=UART_RX[uart].rd_ptr;
	wr_pos=UART_RX[uart].wr_ptr;
	// check if there is data in buffer
	if(rd_pos!=wr_pos) {
	// read data
	wert1 = UART_RX[uart].rx_buffer[rd_pos];
	// increase read pointer
	rd_pos++;
	count++;
	if(rd_pos>=RX_BUF_SIZE) rd_pos=0;

	wert2 = UART_RX[uart].rx_buffer[rd_pos];
	rd_pos++;
	count++;
	if(rd_pos>=RX_BUF_SIZE) rd_pos=0;

	wert3 = UART_RX[uart].rx_buffer[rd_pos];
	rd_pos++;
	count++;
	if(rd_pos>=RX_BUF_SIZE) rd_pos=0;

	wert4 = UART_RX[uart].rx_buffer[rd_pos];
	rd_pos++;
	count++;
	if(rd_pos>=RX_BUF_SIZE) rd_pos=0;

		if(endianness == BIG_ENDIAN_T){
			data[0] = wert1;
			data[1] = wert2;
			data[2] = wert3;
			data[3] = wert4;
		}
		else if(endianness == LITTLE_ENDIAN_T){
			data[3] = wert1;
			data[2] = wert2;
			data[1] = wert3;
			data[0] = wert4;
		}
	}
	if(rd_pos > wr_pos) return 0;
    // update read pointer
    UART_RX[uart].rd_ptr=rd_pos;
    return count;
}

uint8_t uart_Receive1Byte(UART_NAME_t uart, uint8_t *data)
{
	uint8_t count = 0;
  uint16_t rd_pos,wr_pos;

  // read current pointer
  rd_pos=UART_RX[uart].rd_ptr;
  wr_pos=UART_RX[uart].wr_ptr;

  // check if there is data in buffer
  if(rd_pos!=wr_pos) {
    // read data
	 *data = UART_RX[uart].rx_buffer[rd_pos];
	// increase read pointer
	rd_pos++;
	count++;
	if(rd_pos>=RX_BUF_SIZE) rd_pos=0;
  }
  // update read pointer
  UART_RX[uart].rd_ptr=rd_pos;
  return count;
}

uint16_t get_rd_ptr(UART_NAME_t uart){
	return UART_RX[uart].rd_ptr;
}

uint16_t get_wr_ptr(UART_NAME_t uart){
	return UART_RX[uart].wr_ptr;
}

void reset_ringbuffer(UART_NAME_t uart){
	UART_RX[uart].wr_ptr = 0;
	UART_RX[uart].rd_ptr = 0;
}

//--------------------------------------------------------------
// internal function
// store received byte in buffer
//--------------------------------------------------------------
void P_UART_Receive(UART_NAME_t uart, uint16_t wert)
{
  uint16_t wr_pos;

  // store byte at write position
  wr_pos=UART_RX[uart].wr_ptr;

  UART_RX[uart].rx_buffer[wr_pos]=wert;
  // increase read pointer
  wr_pos++;
  if(wr_pos>=RX_BUF_SIZE) wr_pos=0;
  UART_RX[uart].wr_ptr=wr_pos;

  // check for overflow
  if(UART_RX[uart].wr_ptr==UART_RX[uart].rd_ptr) {
    UART_RX[uart].status=RX_FULL;
  }
}

//--------------------------------------------------------------
// internal function
// UART-Interrupt-Function
// Interrupt-Nr must be given as parameter int_nr
//--------------------------------------------------------------
void P_UART_RX_INT(uint8_t int_nr, uint16_t wert)
{
  UART_NAME_t nr;

  // search for fitting entry
  for(nr=0;nr<UART_ANZ;nr++) {
    if(UART[nr].INT==int_nr) {
      // entry found, store byte
      P_UART_Receive(nr,wert);
      break;
    }
  }
}
 /***********************************************************
 * Function definitions
 ***********************************************************/
//--------------------------------------------------------------
// send a byte via UART
//--------------------------------------------------------------
void uart_SendByte(UART_NAME_t uart, uint16_t wert)
{
  // wait till previous Byte is sent
  while (USART_GetFlagStatus(UART[uart].UART, USART_FLAG_TXE) == RESET);
  USART_SendData(UART[uart].UART, wert);
}

 /***********************************************************
 * Definition internal functions
 ***********************************************************/
//--------------------------------------------------------------
// UART1-Interrupt
//--------------------------------------------------------------
void USART1_IRQHandler(void) {
  uint16_t wert;

  if (USART_GetITStatus(USART1, USART_IT_RXNE) == SET) {
    // when there is a byte in receive buffer
    wert=USART_ReceiveData(USART1);
    // store byte
    P_UART_RX_INT(USART1_IRQn,wert);
  }
}

//--------------------------------------------------------------
// UART3-Interrupt
//--------------------------------------------------------------
void USART3_IRQHandler(void) {
  uint16_t wert;

  if (USART_GetITStatus(USART3, USART_IT_RXNE) == SET) {
    // when there is a byte in receive buffer
    wert=USART_ReceiveData(USART3);
    // store byte
    P_UART_RX_INT(USART3_IRQn,wert);
  }
}

//--------------------------------------------------------------
// UART6-Interrupt
//--------------------------------------------------------------
void USART6_IRQHandler(void) {
  uint16_t wert;

  if (USART_GetITStatus(USART6, USART_IT_RXNE) == SET) {
    // when there is a byte in receive buffer
    wert=USART_ReceiveData(USART6);
    // store byte
    P_UART_RX_INT(USART6_IRQn,wert);
  }
}


//--------------------------------------------------------------
// UART7-Interrupt
//--------------------------------------------------------------
void UART7_IRQHandler(void) {
  uint16_t wert;

  if (USART_GetITStatus(UART7, USART_IT_RXNE) == SET) {
    // when there is a byte in receive buffer
    wert=USART_ReceiveData(UART7);
    // store byte
    P_UART_RX_INT(UART7_IRQn,wert);
  }
}
