/***********************************************************
 *
 * d_SDRAM_FLASH.h
 * SYSTART GmbH
 * SAM, 23.06.2017
 *
 ***********************************************************/

#ifndef __D_SDRAM_FLASH_H_
#define __D_SDRAM_FLASH_H_

/***********************************************************
 * Includes
 ***********************************************************/

#include "stm32f4xx.h"
#include "global_Memory.h"

/***********************************************************
 * Funktionsdeklarationen
 ***********************************************************/
/**
* @brief Initialisierung des externen SDRAMs und NOR-Flashs
*
* -
*
* @param -
*
* @return -
*
*/
void SDRAM_FLASH_init ( void );

/**
* @brief Schreibt einen 8bit-Wert an die uebergebene SDRAM-Speicheradresse
*
* -
*
* @param adr 		Adresse
* @param data 		zu speichernde Daten
*
* @return -
*
*/
void SDRAM_Write_8b ( uint32_t adr, uint8_t data );

/**
* @brief Schreibt einen 16bit-Wert an die uebergebene SDRAM-Speicheradresse
*
* -
*
* @param adr 		Adresse
* @param data 		zu speichernde Daten
*
* @return -
*
*/
void SDRAM_Write_16b ( uint32_t adr, uint16_t data );

/**
* @brief Schreibt einen 32bit-Wert an die uebergebene SDRAM-Speicheradresse
*
* -
*
* @param adr 		Adresse
* @param data 		zu speichernde Daten
*
* @return -
*
*/
void SDRAM_Write_32b ( uint32_t adr, uint32_t data );

/**
* @brief Liest einen 8bit-Wert von der uebergebenen SDRAM-Speicheradresse
*
* -
*
* @param adr 		Adresse
*
* @return uint8_t	ausgelesener Wert
*
*/
uint8_t SDRAM_Read_8b ( uint32_t adr );

/**
* @brief Liest einen 16bit-Wert von der uebergebenen SDRAM-Speicheradresse
*
* -
*
* @param adr 		Adresse
* @param data 		Pointer fuer ausgelesene Daten
*
* @return uint16_t	ausgelesener Wert
*
*/
uint16_t SDRAM_Read_16b ( uint32_t adr );

/**
* @brief Liest 32bit-Wert von der uebergebenen SDRAM-Speicheradresse
*
* -
*
* @param adr 		Adresse
* @param data 		Pointer fuer ausgelesene Daten
*
* @return uint32_t	ausgelesener Wert
*
*/
uint32_t SDRAM_Read_32b ( uint32_t adr );

/**
* @brief ID des NOR-Flashs auslesen
*
* Manufacturer and Device Code
*
* @param NOR_ID 	Pointer fuer ausgelesene Daten
*
* @return -
*
*/
void NOR_ReadID ( NOR_IDTypeDef* NOR_ID );

/**
* @brief Loeschen eines NOR Speicherblocks
*
* Block: 128kByte
*
* @param BlockAddr 	Adresse des zu loeschenden Blocks
*
* @return -
*
*/
NOR_Status NOR_EraseBlock ( uint32_t BlockAddr );

/**
* @brief Loeschen des gesamten NOR Flashs
*
* -
*
* @param -
*
* @return -
*
*/
NOR_Status NOR_EraseChip ( void );

/**
* @brief Schreiben eines Buffer mit 16b-Werten ueber Puffer-Schreiben
*
* -
*
* @param pBuffer 				Pointer auf Buffer mit 16b-Werten
* @param WriteAddr 				Adresse fuer den Buffer im NOR-Flash
* @param NumHalfwordToWrite 	Groesse des zu schreibenden Buffers
*
* @return NOR_Status			Status des NOR-Flashs
*
*/
NOR_Status NOR_ProgramBuffer ( uint16_t* pBuffer, uint32_t WriteAddr, uint32_t NumHalfwordToWrite );

/**
* @brief Schreiben eines 16b-Wertes in das NOR-Flash
*
* -
*
* @param WriteAddr 		Adresse an die der 16b-Wert geschrieben werden soll
* @param Data			16b-Wert
*
* @return NOR_Status	Status des NOR-Flashs
*
*/
NOR_Status NOR_Write_16b ( uint32_t WriteAddr, uint16_t Data );

/**
* @brief Schreiben mehrerer 16b-Werte in das NOR-Flash
*
* In dieser Funktion wird mehrmals die Funktion NOR_Write_16b
* aufgerufen. Damit ist diese Funktion langsamer als die
* NOR_ProgramBuffer-Funktion
*
* @param pBuffer			Pointer auf die zu schreibenden 16b-Werte
* @param WriteAddr 			Adresse an die der 16b-Werte geschrieben werden soll
* @param NumToWrite			Anzahl der zu schreibenden 16b-Werte
*
* @return NOR_Status	Status des NOR-Flashs
*
*/
NOR_Status NOR_WriteBuffer_16b ( uint16_t* pBuffer, uint32_t WriteAddr, uint32_t NumToWrite );

/**
* @brief Lesen eines 16b-Wertes aus dem NOR-Flash
*
* -
*
* @param ReadAddr 		Adresse aus der ausgelesen werden soll
*
* @return uint16_t		ausgelesener 16b-Wert
*
*/
uint16_t NOR_Read_16b ( uint32_t ReadAddr );

/**
* @brief Lesen mehrerer 16b-Werte aus dem NOR-Flash
*
* Es wird mehrmals die Funktion NOR_Read_16b aufgerufen.
*
* @param pBuffer		Pointer auf die zu lesenden 16b-Werte
* @param ReadAddr 		Adresse ab die der 16b-Werte gelesen werden sollen
* @param NumToRead		Anzahl der zu lesenden 16b-Werte
*
* @return -
*
*/
void NOR_ReadBuffer_16b (uint16_t* pBuffer, uint32_t ReadAddr, uint32_t NumToRead );


/**
* @brief NOR-Flash in Lese-Modus versetzen
*
* -
*
* @param -
*
* @return NOR_Status	Status des NOR-Flashs
*
*/
NOR_Status NOR_ReturnToReadMode ( void );

/**
* @brief NOR-Flash Fehlerregister zuruecksetzen und in Lese-Modus versetzen
*
* -
*
* @param -
*
* @return NOR_Status	Status des NOR-Flashs
*
*/
NOR_Status NOR_Reset ( void );

/**
* @brief Status des NOR-Flashs auslesen
*
* -
*
* @param Timeout		Timeout zum NOR-Flash programmieren
*
* @return NOR_Status	Status des NOR-Flashs
*
*/
NOR_Status NOR_GetStatus ( uint32_t Timeout );

#endif // __D_SDRAM_H_
