/***********************************************************
 *
 * d_uart.h
 * SYSTART GmbH
 * FLB, 12.09.2018
 *
 ***********************************************************/
#ifndef DRIVER_D_UART_H_
#define DRIVER_D_UART_H_

/***********************************************************
 * Includes
 ***********************************************************/
#include "stm32f4xx.h"

/***********************************************************
 * Defines
 ***********************************************************/
//--------------------------------------------------------------
// List of all UARTs
// (start with 0)
//--------------------------------------------------------------
typedef enum
{
  COM1 = 0,  // COM1 (TX=PA9, RX=PA10)
  COM3 = 1,  // COM3 (TX= PB10, RX=PB11)
  COM6 = 2   // COM6 (TX=PC6, RX=PC7)
}UART_NAME_t;

#define  UART_ANZ   3 // Number of UART_NAME_t

//--------------------------------------------------------------
// end command for transfer
//--------------------------------------------------------------
typedef enum {
  NONE = 0,  // no end command
  LFCR,      // LineFeed + CarriageReturn (0x0A,0x0D)
  CRLF,      // CarriageReturn + LineFeed (0x0D,0x0A)
  LF,        // only LineFeed (0x0A)
  CR         // only CarriageReturn (0x0D)
}UART_LASTBYTE_t;

//--------------------------------------------------------------
// Reception state
//--------------------------------------------------------------
typedef enum {
  RX_EMPTY = 0,  // nothing received
  RX_READY,      // data is in buffer
  RX_FULL        // RX-Puffer is full
}UART_RXSTATUS_t;

//--------------------------------------------------------------
// struct of a UART-Pin
//--------------------------------------------------------------
typedef struct {
  GPIO_TypeDef* PORT;     // Port
  const uint16_t PIN;     // Pin
  const uint32_t CLK;     // Clock
  const uint8_t SOURCE;   // Source
}UART_PIN_t;

//--------------------------------------------------------------
// struct of UARTs
//--------------------------------------------------------------
typedef struct {
  UART_NAME_t UART_NAME;    // Name
  const uint32_t CLK;       // Clock
  const uint8_t AF;         // AF
  USART_TypeDef* UART;      // UART
  const uint32_t BAUD;      // Baudrate
  const uint8_t INT;        // Interrupt
  UART_PIN_t TX;            // TX-Pin
  UART_PIN_t RX;            // RX-Pin
}UART_t;

//--------------------------------------------------------------
// Defines for reception
//--------------------------------------------------------------
#define  RX_BUF_SIZE   800   // Size of RX-buffer in Bytes
#define  RX_FIRST_CHR  0x00//0x20  // first allowed character (ASCII-value)
#define  RX_LAST_CHR   0xFF//0x7E  // last allowed character (ASCII-value)
#define  RX_END_CHR    0x0A  // end command (ASCII-value) = \n


//Defines for Endianness
typedef enum{
	BIG_ENDIAN_T,
	LITTLE_ENDIAN_T
}Endianness_t;

//--------------------------------------------------------------
// struct for UART_RX
//--------------------------------------------------------------
typedef struct {
  char rx_buffer[RX_BUF_SIZE]; // RX-buffer (Ring-buffer)
  uint16_t wr_ptr;             // Write Pointer
  uint16_t rd_ptr;             // Read Pointer
  UART_RXSTATUS_t status;      // RX-State
}UART_RX_t;
UART_RX_t UART_RX[UART_ANZ];

/***********************************************************
 * Function declarations
************************************************************/
void uart_init(void);
void uart_SendByte(UART_NAME_t uart, uint16_t wert);
void uart_SendString(UART_NAME_t uart, char *ptr, UART_LASTBYTE_t end_cmd);
UART_RXSTATUS_t uart_ReceiveString(UART_NAME_t uart, char *ptr);
void uart_SendArray(UART_NAME_t uart, uint8_t *data, uint16_t cnt, Endianness_t endianness);
uint32_t uart_ReceiveArray(UART_NAME_t uart, uint8_t *data);
uint8_t uart_Receive4Byte(UART_NAME_t uart, uint8_t *data, Endianness_t endianness);
uint8_t uart_Receive2Byte(UART_NAME_t uart, uint16_t *data, Endianness_t endianness);
uint8_t uart_Receive1Byte(UART_NAME_t uart, uint8_t *data);
uint16_t get_rd_ptr(UART_NAME_t uart);
uint16_t get_wr_ptr(UART_NAME_t uart);
void reset_ringbuffer(UART_NAME_t uart);

#endif /* DRIVER_D_UART_H_ */
