
/***********************************************************
 * Includes
 ***********************************************************/
#include "stm32f4xx.h"
/* Driver */
#include "driver/d_uart.h"
#include "driver/d_timer.h"
#include "driver/d_Backlight.h"
#include "driver/d_Display_DMA2D.h"
 /* Applications */
#include "application/uart_esp32_comm.h"
#include "application/weather_interface.h"

#if 0
#ifdef __GNUC__
  /* With GCC/RAISONANCE, small printf (option LD Linker->Libraries->Small printf
     set to 'Yes') calls __io_putchar() */
  #define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
  #define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
#endif /* __GNUC__ */
#endif /* For Debugging */

/***********************************************************
 * Private Function Prototypes
 ***********************************************************/
void HW_Init(void);
void display_init(void);
void gpio_init(void);
void status_led_blink(void);




static weather_data_struct weather_data;

/*****************************************************************
 *
 * Function: int main(void)
 *
 * Summary: Entry point of display control application
 *
 * Parameters:
 *
 * Return:
 *
 * Revision History:
 * Creation Date: 21062021
 *
 *****************************************************************/
int main(void) {

	/* Initialize the peripherals */
	HW_Init();
	//input_Init();
	display_init();

	comm_err_t err = check_communication_esp32();
	if (err != FCN_OK){
		// communication with esp32 module fails
		while(1){
			status_led_blink();
		}
	}

	get_ip_address_esp32(&weather_data);
	draw_weather_interface(weather_data.ip_address);
	for(;;){
		send_cmd_esp32(GET_ALL);
		get_data_esp32(&weather_data);
		draw_weather_data(weather_data);
		delay_ms(1000);
	}
	return 0;
}


/*****************************************************************
 *
 * Function:void HW_Init(void)
 *
 * Summary: Various Hardware initializations
 *
 * Parameters:
 *
 * Return:
 *
 * Revision History:
 * Creation Date: 08042019
 *
 *****************************************************************/
void HW_Init(void) {
	SystemInit();

	timer_init();
	uart_init();

	Backlight_init();
	Backlight_configPWM(95);
	Backlight_ON();

	gpio_init();
}

/*****************************************************************
 *
 * Function:  void display_init(void)
 *
 * Summary:   Initializes the display and populate the pic and font
 * 			   arrays
 *
 * Parameters:
 *
 * Return:
 *
 * Revision History:
 * Creation Date: 08042019
 *
 *****************************************************************/
void display_init(void) {

	SDRAM_FLASH_init();
	DMA2D_Display_init();

	switch_buffer(Layer_1, Buffer_1);
	DMA2D_Fill_Color(0xFFFFFFFF, Layer_1, Buffer_1);

	uint16_t image_width = ebssystart_logo_35.width;
	uint16_t image_height = ebssystart_logo_35.height;
	DMA2D_Draw_Image((HDP-image_width)/2, (VDP-image_height)/2, image_width, image_height, 0xFF, REPLACE_ALPHA_VALUE, (uint32_t)ebssystart_logo_35.data, CM_ARGB8888, Layer_1, Buffer_1);

	delay_ms(4000);
	image_width = twoInone_display_logo.width;
	image_height = twoInone_display_logo.height;
	DMA2D_Fill_Color(0xFFFFFFFF, Layer_1, Buffer_1);
	DMA2D_Draw_Image((HDP-image_width)/2, (VDP-image_height)/2, image_width, image_height, 0xFF, REPLACE_ALPHA_VALUE, (uint32_t)twoInone_display_logo.data, CM_ARGB8888, Layer_1, Buffer_1);

	delay_ms(4000);
}



/*****************************************************************
 *
 * Function: void gpio_init(void)
 *
 * Summary: Configure GPIOs for D_BUSY and User LEDs
 *
 * Parameters:
 *
 * Return:
 *
 * Revision History:
 * Creation Date: 30052019
 *
 *****************************************************************/
void gpio_init(void) {
	GPIO_InitTypeDef GPIO_InitStruct;

	// Clocks activation
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOA, ENABLE );
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOB, ENABLE );
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOK, ENABLE );

	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStruct.GPIO_Speed = GPIO_Speed_2MHz;

	// PA6 = USER_LED1, PA5 = USER_LED2
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_5;
	GPIO_Init( GPIOA, &GPIO_InitStruct);
	GPIO_SetBits(GPIOA, GPIO_Pin_5 | GPIO_Pin_6);

	//PK7 = DE
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_7;
	GPIO_InitStruct.GPIO_OType = GPIO_OType_OD;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_DOWN;
	GPIO_Init( GPIOK, &GPIO_InitStruct);
	GPIO_ResetBits(GPIOK, GPIO_Pin_7);

}

/*****************************************************************
 *
 * Function: void status_led_blink(void)
 *
 * Summary: Blink the user LEDs
 *
 * Parameters:
 *
 * Return:
 *
 * Revision History:
 * Creation Date: 30052019
 *
 *****************************************************************/
void status_led_blink(void) {
	static tTimer timeout = 0;

	if (timer_get_time_ms() - timeout > 1000) {
		GPIO_ToggleBits(GPIOA, GPIO_Pin_6);
		GPIO_ToggleBits(GPIOA, GPIO_Pin_5);
		timeout = timer_get_time_ms();
	}
}


#if 0 //For Debugging
PUTCHAR_PROTOTYPE
{
  USART_SendData(UART[0].UART, (uint8_t) ch);

  /* Loop until the end of transmission */
  while (USART_GetFlagStatus(UART[0].UART, USART_FLAG_TC) == RESET)
  {}

  return ch;
}
#endif
